@@ -104,10 +104,14 @@ pub use char::{
 pub use combinator::{
     between,
     chainl1,
+    choice_slice,
+    choice_vec,
     many,
     many1,
     optional,
     sep_by,
+    skip_many,
+    skip_many1,
     try,
     value,
     unexpected,
@@ -117,7 +121,9 @@ pub use combinator::{
 };
 
 macro_rules! static_fn {
-    (($($arg: pat, $arg_ty: ty),*) -> $ret: ty { $body: expr }) => { { fn temp($($arg: $arg_ty),*) -> $ret { $body } temp } }
+    (($($arg: pat, $arg_ty: ty),*) -> $ret: ty { $body: expr }) => { {
+        fn temp($($arg: $arg_ty),*) -> $ret { $body } temp as fn (_) -> _
+    } }
 }
 
 ///Module containing the primitive types which is used to create and compose more advanced parsers
@@ -131,11 +137,13 @@ pub mod char;
 mod tests {
     use super::*;
     use super::primitives::{SourcePosition, State, Stream, Error, Consumed};
+    use std::borrow::IntoCow;
     
 
     fn integer<'a, I>(input: State<I>) -> ParseResult<i64, I>
         where I: Stream<Item=char> {
         let (s, input) = try!(many1::<String, _>(digit())
+            .expected("integer")
             .parse_state(input));
         let mut n = 0;
         for c in s.chars() {
@@ -200,15 +208,19 @@ r"
         Times(Box<Expr>, Box<Expr>),
     }
     fn expr(input: State<&str>) -> ParseResult<Expr, &str> {
-        let word = many1(satisfy(|c| c.is_alphabetic()));
+        let word = many1(satisfy(|c| c.is_alphabetic()))
+            .expected("identifier");
         let integer = integer as fn (_) -> _;
-        let array = between(satisfy(|c| c == '['), satisfy(|c| c == ']'), sep_by(expr as fn (_) -> _, satisfy(|c| c == ',')));
+        let array = between(satisfy(|c| c == '['), satisfy(|c| c == ']'), sep_by(expr as fn (_) -> _, satisfy(|c| c == ',')))
+            .expected("[");
+        let paren_expr = between(satisfy(|c| c == '('), satisfy(|c| c == ')'), term as fn (_) -> _)
+            .expected("(");
         let spaces = spaces();
         spaces.clone().with(
                 word.map(Expr::Id)
                 .or(integer.map(Expr::Int))
                 .or(array.map(Expr::Array))
-                .or(between(satisfy(|c| c == '('), satisfy(|c| c == ')'), term as fn (_) -> _))
+                .or(paren_expr)
             ).skip(spaces)
             .parse_state(input)
     }
@@ -235,17 +247,40 @@ r"
             .parse(input);
         let err = ParseError {
             position: SourcePosition { line: 2, column: 1 },
-            errors: vec![Error::Unexpected(','), Error::Message("Expected digit".to_string())]
+                errors: vec![
+                    Error::Unexpected(','),
+                    Error::Expected("identifier".into_cow()),
+                    Error::Expected("integer".into_cow()),
+                    Error::Expected("[".into_cow()),
+                    Error::Expected("(".into_cow()),
+                ]
         };
         assert_eq!(result, Err(err));
     }
 
+    #[test]
+    fn expression_error_message() {
+        let input =
+r"
+,123
+";
+        let result = (expr as fn (_) -> _)
+            .parse(input);
+        let m = format!("{}", result.unwrap_err());
+let expected =
+r"Parse error at line: 2, column: 1
+Unexpected character ','
+Expected 'identifier', 'integer', '[' or '('
+";
+        assert_eq!(m, expected);
+    }
+
     fn term(input: State<&str>) -> ParseResult<Expr, &str> {
 
         let mul = satisfy(|c| c == '*')
-            .map(|_| Box::new(|&mut:l, r| Expr::Times(Box::new(l), Box::new(r))) as Box<FnMut(_, _) -> _>);
+            .map(|_| Box::new(|l, r| Expr::Times(Box::new(l), Box::new(r))) as Box<FnMut(_, _) -> _>);
         let add = satisfy(|c| c == '+')
-            .map(|_| Box::new(|&mut:l, r| Expr::Plus(Box::new(l), Box::new(r))) as Box<FnMut(_, _) -> _>);
+            .map(|_| Box::new(|l, r| Expr::Plus(Box::new(l), Box::new(r))) as Box<FnMut(_, _) -> _>);
         let factor = chainl1(expr as fn (_) -> _, mul);
         chainl1(factor, add)
             .parse_state(input)
@@ -358,4 +393,23 @@ r"(3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
         let borrow_parser = &mut *parser;
         assert_eq!(borrow_parser.parse("1"), Ok(('1', "")));
     }
+
+    #[test]
+    fn choice_strings() {
+        let mut fruits = [
+            try(string("Apple")),
+            try(string("Banana")),
+            try(string("Cherry")),
+            try(string("Date")),
+            try(string("Fig")),
+            try(string("Grape")),
+        ];
+        let mut parser = choice_slice(fruits.as_mut_slice());
+        assert_eq!(parser.parse("Apple"), Ok(("Apple", "")));
+        assert_eq!(parser.parse("Banana"), Ok(("Banana", "")));
+        assert_eq!(parser.parse("Cherry"), Ok(("Cherry", "")));
+        assert_eq!(parser.parse("DateABC"), Ok(("Date", "ABC")));
+        assert_eq!(parser.parse("Fig123"), Ok(("Fig", "123")));
+        assert_eq!(parser.parse("GrapeApple"), Ok(("Grape", "Apple")));
+    }
 }
